import numpy as np
import matplotlib.pyplot as plt
import seaborn as sb
import scipy.stats as ss
import pystan
import pickle

#
#           データの読み込み
#
flnm = input('Data file name(*.csv) = ')
fin = open(flnm, 'r')
data = fin.readlines()
fin.close()

N = len(data) - 1
DataID = []
RTs = []
for i in range(N):
    str = data[i+1].split(',')
    DataID.append(str[0])
    RTs.append(float(str[1]))

for i in range(N):
    print(DataID[i],'  ', RTs[i])


RTs = np.array(RTs)

xi_hat = RTs.min()          #   シフト値の最大値
print('xi_hat = ', xi_hat)
RTs_xi = RTs - xi_hat
#
#       モーメント法に依るパラメータ値の推定
#
beta_hat = RTs_xi.mean() / RTs_xi.var()
print('beta_hat = ', beta_hat)
alpha_hat = beta_hat * RTs_xi.mean()
print('alpha_hat = ', alpha_hat)

max_a = alpha_hat * 100.0
max_b = beta_hat * 100.0
max_x = xi_hat
Data = {'N': len(RTs), 'RT': RTs, 'max_a': max_a, 'max_b': max_b, 'max_x': max_x}

sm = pystan.StanModel(file = 'shiftedGamma.stan')   

fit = sm.sampling(data = Data, n_jobs = 1)
print(fit)

param_a = fit['param_a']
param_b = fit['param_b']
param_x = fit['param_x']
plt.hist(param_a)
plt.xlabel(r'$\alpha$', fontsize = 14)
plt.title(r'Posterior Distribution of $\alpha$', fontsize = 18)
plt.show()

plt.hist(param_b)
plt.xlabel(r'$\beta$', fontsize = 14)
plt.title(r'Posterior Distribution of $\beta$', fontsize = 18)
plt.show()

plt.hist(param_x)
plt.xlabel(r'$\xi$', fontsize = 14)
plt.title(r'Posterior Distribution of $\xi$', fontsize = 18)
plt.show()

#
#       中央値
#
med_a = np.percentile(param_a, 50)
med_b = np.percentile(param_b, 50)
med_xi = np.percentile(param_x, 50)
print('med_a = ', med_a, '   med_b = ', med_b, '   med_xi = ', med_xi)


min_x = RTs.min() - (RTs.max() - RTs.min()) * 0.1
max_x = RTs.max() + (RTs.max() - RTs.min()) * 0.1
rt = np.arange(min_x, max_x, (max_x - min_x) / 1000)
#
#       ガンマ分布
#
pdf_values = ss.gamma.pdf(rt, med_a, med_xi, 1/med_b)
plt.plot(rt, pdf_values, linewidth = 3, label = 'gamma(ξ,α,β)')
#
#       データのヒストグラム
#
plt.hist(RTs, density = True, label = 'RT')

plt.xlabel('Time', fontsize = 14)
plt.title('Shifted Gamma Distribution\n' +
          'ξ = {0:<.3f}, α = {1:<.3f}, β = {2:<.3f}\n'.format(med_xi, med_a, med_b),
          fontsize = 16)
plt.legend(fontsize = 14)
plt.tight_layout()
plt.show()
